@@ -137,7 +137,7 @@ QString getConfigFile()
 
     //
     // Go through list of files, looking for the preferred one...
-    if (files.count()) {
+    if (!files.isEmpty()) {
         QStringList::const_iterator it(files.begin()),
                     end(files.end());
 
@@ -823,10 +823,10 @@ void KXftConfig::applyAntiAliasing()
 bool KXftConfig::aliasingEnabled()
 {
     FcPattern *pattern = FcPatternCreate();
-    FcConfigSubstitute(0, pattern, FcMatchPattern);
+    FcConfigSubstitute(nullptr, pattern, FcMatchPattern);
     FcDefaultSubstitute(pattern);
     FcResult result;
-    FcPattern *f = FcFontMatch(0, pattern, &result);
+    FcPattern *f = FcFontMatch(nullptr, pattern, &result);
     FcBool antialiased = FcTrue;
     FcPatternGetBool(f, FC_ANTIALIAS, 0, &antialiased);
     FcPatternDestroy(f);
